/************************************************************************
 *   IRC - Internet Relay Chat, doc/basic_aliases.c
 *   Copyright (C) 2003 XNet Development Department
 *   Copyright (C) 2003 Abriel
 *   Copyright (C) 2003 s 
 *   Copyleft  (L) 2003 Sheyh
 *   
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: basic_aliases.c,v 1.0 2003/05/28 01:38:00 Abriel Exp $
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

#define NICKSERV "nickserv"
#define CHANSERV "chanserv"
#define MEMOSERV "memoserv"
#define OPERSERV "operserv"
#define ULINES "services.xnet.org"


static void alias_ns(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void alias_cs(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void alias_ms(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void alias_os(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static char * collect_all_parvs(int parc, char * parv[]);

struct Message nickserv_msgtab = {

  NICKSERV, 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_ns, alias_ns, alias_ns, alias_ns}

};

struct Message ns_msgtab = {

  "NS", 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_ns, alias_ns, alias_ns, alias_ns}

};

struct Message chanserv_msgtab = {

  "CHANSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_cs, alias_cs, alias_cs, alias_cs}

};

struct Message cs_msgtab = {

  "CS", 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_cs, alias_cs, alias_cs, alias_cs}

};

struct Message memoserv_msgtab = {

  "MEMOSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_ms, alias_ms, alias_ms, alias_ms}

};

struct Message ms_msgtab = {

  "MS", 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_ms, alias_ms, alias_ms, alias_ms}

};

struct Message operserv_msgtab = {

  "OPERSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_os, alias_os, alias_os, alias_os}

};

struct Message os_msgtab = {

  "OS", 0, 0, 1, 0, MFLG_SLOW, 0,
  {alias_os, alias_os, alias_os, alias_os}

};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&nickserv_msgtab);
  mod_add_cmd(&ns_msgtab);
  mod_add_cmd(&chanserv_msgtab);
  mod_add_cmd(&cs_msgtab);
  mod_add_cmd(&memoserv_msgtab);
  mod_add_cmd(&ms_msgtab);
  mod_add_cmd(&operserv_msgtab);
  mod_add_cmd(&os_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&nickserv_msgtab);
  mod_del_cmd(&ns_msgtab);
  mod_del_cmd(&chanserv_msgtab);
  mod_del_cmd(&cs_msgtab);
  mod_del_cmd(&memoserv_msgtab);
  mod_del_cmd(&ms_msgtab);
  mod_del_cmd(&operserv_msgtab);
  mod_del_cmd(&os_msgtab);
}

const char *_version = "$Revision: 1.1 $";
#endif

static void alias_ns(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
    char * all_parvs;
    if(find_person("NickServ"))    
    {
       if(IsOper(find_person("NickServ")))
       {
         all_parvs = collect_all_parvs(parc, parv);
         sendto_anywhere(find_person("NickServ"), source_p, "PRIVMSG NickServ :%s", collect_all_parvs(parc, parv));
         MyFree(all_parvs);
       }
    }
    else
    {
       sendto_one(source_p, ":%s NOTICE %s :Services are currently down, try again later", me.name); 
    }
}

static void alias_cs(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
    char * all_parvs;
    if(find_person("ChanServ"))
    {
       if(IsOper(find_person("ChanServ")))
       {
         all_parvs = collect_all_parvs(parc, parv);
         sendto_anywhere(find_person("ChanServ"), source_p, "PRIVMSG ChanServ :%s", collect_all_parvs(parc, parv));
         MyFree(all_parvs);
       }
    }
    else
    {
       sendto_one(source_p, ":%s NOTICE %s :Services are currently down, try again later", me.name);
    }
}

static void alias_ms(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
    char * all_parvs;
    if(find_person("MemoServ"))
    {
       if(IsOper(find_person("MemoServ")))  
       {
         all_parvs = collect_all_parvs(parc, parv);
         sendto_anywhere(find_person("MemoServ"), source_p, "PRIVMSG MemoServ :%s", collect_all_parvs(parc, parv));
         MyFree(all_parvs);
       }
    }
    else
    {
       sendto_one(source_p, ":%s NOTICE %s :Services are currently down, try again later", me.name);
    }
}

static void alias_os(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
    char * all_parvs;
    if(find_person("OperServ"))
    {
       if(IsOper(find_person("OperServ")))
       {
         all_parvs = collect_all_parvs(parc, parv);
         sendto_anywhere(find_person("OperServ"), source_p, "PRIVMSG OperServ :%s", collect_all_parvs(parc, parv));
         MyFree(all_parvs);
       }
    }
    else
    {
       sendto_one(source_p, ":%s NOTICE %s :Services are currently down, try again later", me.name);
    }
}

static char * collect_all_parvs(int parc, char * parv[])
{
  int len = 0;
  char * buffer;
  int i;

  for(i = 0; i < parc; i++)
  {
    len += strlen(parv[i]);
  }
  buffer = (char *) MyMalloc(sizeof(char) * len + 1);
  if (parc)
  {
     sprintf(buffer, "%s", parv[1]);
     for(i = 2; i < parc; i++)
     {
       sprintf(buffer, "%s %s", buffer, parv[i]);
     }
  }

  return buffer;
}

